@@ -13,76 +13,94 @@
 # limitations under the License.
 import argparse
 import os.path
+from typing import Dict
+from typing import List
+from typing import cast
 
-from twine.commands import _find_dists
-from twine.package import PackageFile
+import requests
+
+from twine import commands
 from twine import exceptions
+from twine import package as package_file
 from twine import settings
 from twine import utils
 
 
-def skip_upload(response, skip_existing, package):
-    filename = package.basefilename
-    msg_400 = (
-        # Old PyPI message:
-        f'A file named "{filename}" already exists for',
-        # Warehouse message:
-        'File already exists',
-        # Nexus Repository OSS message:
-        'Repository does not allow updating assets',
-    )
-    msg_403 = 'Not enough permissions to overwrite artifact'
+def skip_upload(
+    response: requests.Response, skip_existing: bool, package: package_file.PackageFile
+) -> bool:
+    if not skip_existing:
+        return False
+
+    status = response.status_code
+    reason = getattr(response, "reason", "").lower()
+    text = getattr(response, "text", "").lower()
+
     # NOTE(sigmavirus24): PyPI presently returns a 400 status code with the
     # error message in the reason attribute. Other implementations return a
-    # 409 or 403 status code. We only want to skip an upload if:
-    # 1. The user has told us to skip existing packages (skip_existing is
-    #    True) AND
-    # 2. a) The response status code is 409 OR
-    # 2. b) The response status code is 400 AND it has a reason that matches
-    #       what we expect PyPI or Nexus OSS to return to us. OR
-    # 2. c) The response status code is 403 AND the text matches what we
-    #       expect Artifactory to return to us.
-    return (skip_existing and (response.status_code == 409 or
-            (response.status_code == 400 and
-             response.reason.startswith(msg_400)) or
-            (response.status_code == 403 and msg_403 in response.text)))
+    # 403 or 409 status code.
+    return (
+        # pypiserver (https://pypi.org/project/pypiserver)
+        status == 409
+        # PyPI / TestPyPI
+        or (status == 400 and "already exist" in reason)
+        # Nexus Repository OSS (https://www.sonatype.com/nexus-repository-oss)
+        or (status == 400 and "updating asset" in reason)
+        # Artifactory (https://jfrog.com/artifactory/)
+        or (status == 403 and "overwrite artifact" in text)
+    )
 
 
-def upload(upload_settings, dists):
-    dists = _find_dists(dists)
+def _make_package(
+    filename: str, signatures: Dict[str, str], upload_settings: settings.Settings
+) -> package_file.PackageFile:
+    """Create and sign a package, based off of filename, signatures and settings."""
+    package = package_file.PackageFile.from_filename(filename, upload_settings.comment)
 
+    signed_name = package.signed_basefilename
+    if signed_name in signatures:
+        package.add_gpg_signature(signatures[signed_name], signed_name)
+    elif upload_settings.sign:
+        package.sign(upload_settings.sign_with, upload_settings.identity)
+
+    if upload_settings.verbose:
+        file_size = utils.get_file_size(package.filename)
+        print(f"  {package.filename} ({file_size})")
+        if package.gpg_signature:
+            print(f"  Signed with {package.signed_filename}")
+
+    return package
+
+
+def upload(upload_settings: settings.Settings, dists: List[str]) -> None:
+    dists = commands._find_dists(dists)
     # Determine if the user has passed in pre-signed distributions
     signatures = {os.path.basename(d): d for d in dists if d.endswith(".asc")}
     uploads = [i for i in dists if not i.endswith(".asc")]
-    upload_settings.check_repository_url()
-    repository_url = upload_settings.repository_config['repository']
 
+    upload_settings.check_repository_url()
+    repository_url = cast(str, upload_settings.repository_config["repository"])
     print(f"Uploading distributions to {repository_url}")
 
+    packages_to_upload = [
+        _make_package(filename, signatures, upload_settings) for filename in uploads
+    ]
+
     repository = upload_settings.create_repository()
     uploaded_packages = []
 
-    for filename in uploads:
-        package = PackageFile.from_filename(filename, upload_settings.comment)
-        skip_message = (
-            "  Skipping {} because it appears to already exist".format(
-                package.basefilename)
+    for package in packages_to_upload:
+        skip_message = "  Skipping {} because it appears to already exist".format(
+            package.basefilename
         )
 
         # Note: The skip_existing check *needs* to be first, because otherwise
         #       we're going to generate extra HTTP requests against a hardcoded
         #       URL for no reason.
-        if (upload_settings.skip_existing and
-                repository.package_is_uploaded(package)):
+        if upload_settings.skip_existing and repository.package_is_uploaded(package):
             print(skip_message)
             continue
 
-        signed_name = package.signed_basefilename
-        if signed_name in signatures:
-            package.add_gpg_signature(signatures[signed_name], signed_name)
-        elif upload_settings.sign:
-            package.sign(upload_settings.sign_with, upload_settings.identity)
-
         resp = repository.upload(package)
 
         # Bug 92. If we get a redirect we should abort because something seems
@@ -91,8 +109,7 @@ def upload(upload_settings, dists):
         # redirects as well.
         if resp.is_redirect:
             raise exceptions.RedirectDetected.from_args(
-                repository_url,
-                resp.headers["location"],
+                repository_url, resp.headers["location"],
             )
 
         if skip_upload(resp, upload_settings.skip_existing, package):
@@ -105,7 +122,7 @@ def upload(upload_settings, dists):
 
     release_urls = repository.release_urls(uploaded_packages)
     if release_urls:
-        print('\nView at:')
+        print("\nView at:")
         for url in release_urls:
             print(url)
 
@@ -114,7 +131,7 @@ def upload(upload_settings, dists):
     repository.close()
 
 
-def main(args):
+def main(args: List[str]) -> None:
     parser = argparse.ArgumentParser(prog="twine upload")
     settings.Settings.register_argparse_arguments(parser)
     parser.add_argument(
@@ -122,13 +139,13 @@ def main(args):
         nargs="+",
         metavar="dist",
         help="The distribution files to upload to the repository "
-             "(package index). Usually dist/* . May additionally contain "
-             "a .asc file to include an existing signature with the "
-             "file upload.",
+        "(package index). Usually dist/* . May additionally contain "
+        "a .asc file to include an existing signature with the "
+        "file upload.",
     )
 
-    args = parser.parse_args(args)
-    upload_settings = settings.Settings.from_argparse(args)
+    parsed_args = parser.parse_args(args)
+    upload_settings = settings.Settings.from_argparse(parsed_args)
 
     # Call the upload function with the arguments from the command line
-    return upload(upload_settings, args.dists)
+    return upload(upload_settings, parsed_args.dists)
