@@ -11,29 +11,27 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import collections
 import hashlib
 import io
 import os
 import subprocess
-from hashlib import blake2b
+from typing import Dict
+from typing import NamedTuple
+from typing import Optional
+from typing import Sequence
+from typing import Tuple
+from typing import Union
 
-import pkginfo
 import pkg_resources
+import pkginfo
 
-from twine.wheel import Wheel
-from twine.wininst import WinInst
 from twine import exceptions
-
-try:
-    FileNotFoundError = FileNotFoundError
-except NameError:
-    FileNotFoundError = IOError  # Py2
-
+from twine import wheel
+from twine import wininst
 
 DIST_TYPES = {
-    "bdist_wheel": Wheel,
-    "bdist_wininst": WinInst,
+    "bdist_wheel": wheel.Wheel,
+    "bdist_wininst": wininst.WinInst,
     "bdist_egg": pkginfo.BDist,
     "sdist": pkginfo.SDist,
 }
@@ -47,9 +45,18 @@ DIST_EXTENSIONS = {
     ".zip": "sdist",
 }
 
+MetadataValue = Union[str, Sequence[str]]
+
 
 class PackageFile:
-    def __init__(self, filename, comment, metadata, python_version, filetype):
+    def __init__(
+        self,
+        filename: str,
+        comment: Optional[str],
+        metadata: pkginfo.Distribution,
+        python_version: Optional[str],
+        filetype: Optional[str],
+    ) -> None:
         self.filename = filename
         self.basefilename = os.path.basename(filename)
         self.comment = comment
@@ -57,9 +64,9 @@ class PackageFile:
         self.python_version = python_version
         self.filetype = filetype
         self.safe_name = pkg_resources.safe_name(metadata.name)
-        self.signed_filename = self.filename + '.asc'
-        self.signed_basefilename = self.basefilename + '.asc'
-        self.gpg_signature = None
+        self.signed_filename = self.filename + ".asc"
+        self.signed_basefilename = self.basefilename + ".asc"
+        self.gpg_signature: Optional[Tuple[str, bytes]] = None
 
         hasher = HashManager(filename)
         hasher.hash()
@@ -70,7 +77,7 @@ class PackageFile:
         self.blake2_256_digest = hexdigest.blake2
 
     @classmethod
-    def from_filename(cls, filename, comment):
+    def from_filename(cls, filename: str, comment: Optional[str]) -> "PackageFile":
         # Extract the metadata from the package
         for ext, dtype in DIST_EXTENSIONS.items():
             if filename.endswith(ext):
@@ -78,8 +85,7 @@ class PackageFile:
                 break
         else:
             raise exceptions.InvalidDistribution(
-                "Unknown distribution format: '%s'" %
-                os.path.basename(filename)
+                "Unknown distribution format: '%s'" % os.path.basename(filename)
             )
 
         # If pkginfo encounters a metadata version it doesn't support, it may
@@ -87,10 +93,10 @@ class PackageFile:
         # and version
         if not (meta.name and meta.version):
             raise exceptions.InvalidDistribution(
-                "Invalid distribution metadata. Try upgrading twine if "
-                "possible."
+                "Invalid distribution metadata. Try upgrading twine if possible."
             )
 
+        py_version: Optional[str]
         if dtype == "bdist_egg":
             pkgd = pkg_resources.Distribution.from_filename(filename)
             py_version = pkgd.py_version
@@ -103,17 +109,15 @@ class PackageFile:
 
         return cls(filename, comment, meta, py_version, dtype)
 
-    def metadata_dictionary(self):
+    def metadata_dictionary(self) -> Dict[str, MetadataValue]:
         meta = self.metadata
         data = {
             # identify release
             "name": self.safe_name,
             "version": meta.version,
-
             # file content
             "filetype": self.filetype,
             "pyversion": self.python_version,
-
             # additional meta-data
             "metadata_version": meta.metadata_version,
             "summary": meta.summary,
@@ -133,12 +137,10 @@ class PackageFile:
             "md5_digest": self.md5_digest,
             "sha256_digest": self.sha2_digest,
             "blake2_256_digest": self.blake2_256_digest,
-
             # PEP 314
             "provides": meta.provides,
             "requires": meta.requires,
             "obsoletes": meta.obsoletes,
-
             # Metadata 1.2
             "project_urls": meta.project_urls,
             "provides_dist": meta.provides_dist,
@@ -146,29 +148,28 @@ class PackageFile:
             "requires_dist": meta.requires_dist,
             "requires_external": meta.requires_external,
             "requires_python": meta.requires_python,
-
             # Metadata 2.1
             "provides_extras": meta.provides_extras,
             "description_content_type": meta.description_content_type,
         }
 
         if self.gpg_signature is not None:
-            data['gpg_signature'] = self.gpg_signature
+            data["gpg_signature"] = self.gpg_signature
 
         return data
 
-    def add_gpg_signature(self, signature_filepath, signature_filename):
+    def add_gpg_signature(
+        self, signature_filepath: str, signature_filename: str
+    ) -> None:
         if self.gpg_signature is not None:
-            raise exceptions.InvalidDistribution(
-                'GPG Signature can only be added once'
-            )
+            raise exceptions.InvalidDistribution("GPG Signature can only be added once")
 
         with open(signature_filepath, "rb") as gpg:
             self.gpg_signature = (signature_filename, gpg.read())
 
-    def sign(self, sign_with, identity):
+    def sign(self, sign_with: str, identity: Optional[str]) -> None:
         print(f"Signing {self.basefilename}")
-        gpg_args = (sign_with, "--detach-sign")
+        gpg_args: Tuple[str, ...] = (sign_with, "--detach-sign")
         if identity:
             gpg_args += ("--local-user", identity)
         gpg_args += ("-a", self.filename)
@@ -177,14 +178,15 @@ class PackageFile:
         self.add_gpg_signature(self.signed_filename, self.signed_basefilename)
 
     @classmethod
-    def run_gpg(cls, gpg_args):
+    def run_gpg(cls, gpg_args: Tuple[str, ...]) -> None:
         try:
             subprocess.check_call(gpg_args)
             return
         except FileNotFoundError:
             if gpg_args[0] != "gpg":
                 raise exceptions.InvalidSigningExecutable(
-                    "{} executable not available.".format(gpg_args[0]))
+                    "{} executable not available.".format(gpg_args[0])
+                )
 
         print("gpg executable not available. Attempting fallback to gpg2.")
         try:
@@ -198,7 +200,10 @@ class PackageFile:
             )
 
 
-Hexdigest = collections.namedtuple('Hexdigest', ['md5', 'sha2', 'blake2'])
+class Hexdigest(NamedTuple):
+    md5: Optional[str]
+    sha2: Optional[str]
+    blake2: Optional[str]
 
 
 class HashManager:
@@ -207,59 +212,58 @@ class HashManager:
     This will also allow us to better test this logic.
     """
 
-    def __init__(self, filename):
+    def __init__(self, filename: str) -> None:
         """Initialize our manager and hasher objects."""
         self.filename = filename
+
+        self._md5_hasher = None
         try:
             self._md5_hasher = hashlib.md5()
         except ValueError:
             # FIPs mode disables MD5
-            self._md5_hasher = None
+            pass
 
         self._sha2_hasher = hashlib.sha256()
-        self._blake_hasher = None
-        if blake2b is not None:
-            self._blake_hasher = blake2b(digest_size=256 // 8)
 
-    def _md5_update(self, content):
+        self._blake_hasher = hashlib.blake2b(digest_size=256 // 8)
+
+    def _md5_update(self, content: bytes) -> None:
         if self._md5_hasher is not None:
             self._md5_hasher.update(content)
 
-    def _md5_hexdigest(self):
+    def _md5_hexdigest(self) -> Optional[str]:
         if self._md5_hasher is not None:
             return self._md5_hasher.hexdigest()
         return None
 
-    def _sha2_update(self, content):
+    def _sha2_update(self, content: bytes) -> None:
         if self._sha2_hasher is not None:
             self._sha2_hasher.update(content)
 
-    def _sha2_hexdigest(self):
+    def _sha2_hexdigest(self) -> Optional[str]:
         if self._sha2_hasher is not None:
             return self._sha2_hasher.hexdigest()
         return None
 
-    def _blake_update(self, content):
+    def _blake_update(self, content: bytes) -> None:
         if self._blake_hasher is not None:
             self._blake_hasher.update(content)
 
-    def _blake_hexdigest(self):
+    def _blake_hexdigest(self) -> Optional[str]:
         if self._blake_hasher is not None:
             return self._blake_hasher.hexdigest()
         return None
 
-    def hash(self):
+    def hash(self) -> None:
         """Hash the file contents."""
         with open(self.filename, "rb") as fp:
-            for content in iter(lambda: fp.read(io.DEFAULT_BUFFER_SIZE), b''):
+            for content in iter(lambda: fp.read(io.DEFAULT_BUFFER_SIZE), b""):
                 self._md5_update(content)
                 self._sha2_update(content)
                 self._blake_update(content)
 
-    def hexdigest(self):
+    def hexdigest(self) -> Hexdigest:
         """Return the hexdigest for the file."""
         return Hexdigest(
-            self._md5_hexdigest(),
-            self._sha2_hexdigest(),
-            self._blake_hexdigest(),
+            self._md5_hexdigest(), self._sha2_hexdigest(), self._blake_hexdigest(),
         )
